/*
 * Copyright (c) 2009 Jess Nuevo Chiquero - UAH
 *
 * This file is part of RawVideoTools
 *
 * RawVideoTools is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * RawVideoTools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#include <string.h>

#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <error.h>
#include <errno.h>

#include <fstream>

#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>

#include <dirent.h>
#include <limits>
#include <algorithm>

#include <cv.h>
#include <cxcore.h>
#include <highgui.h>

#include <time.h>

#include <vector>
#include <iostream>

//#include "basic_utils.hh"

#include <boost/smart_ptr.hpp>

#include "frame_header.hh"

int main(int argc, char *argv[])
{

    frame_header head;
    FILE *f=NULL;
    const IplImage *img;
    IplImage *plane = NULL;
    IplImage *aux = NULL;
    uint nframe = 0;
    unsigned int t_offset;

    int header_version = 0;
    struct timeval tv;
   
    bool write_header = false;
    bool reset_frame_counter = false;
    bool convert_color = false;
    CvCapture *capture = NULL;


    std::cout << "avi2raw [input.avi] [output.raw]" << std::endl;

    if(argc<3){
	std::cerr << "No hay suficientes parametros" << std::endl;
	return -1;
    }
	

    if(!strcmp(argv[1], argv[2])){
	std::cerr << "Input and output filenames are the same!"<<std::endl;
	return -1;
    }

    capture = cvCreateFileCapture(argv[1]);
    if(!capture){
	std::cerr << "Bad video file" << std::endl;
	return -1;
    }

    //get 1 image and check if there is a header
    img = cvQueryFrame(capture); 


    int val = check_frame_header(img, 0, &head);

    if(val == -1){
	plane = cvCreateImage(cvGetSize(img), img->depth, 1);
	cvSplit(img, plane, NULL,NULL, NULL);

	val = check_frame_header(plane, 0, &head);	
    }
    
    if(val == -1){ //no header in the images
	write_header = true;		
    }
    else{
	std::cout << "Header found in iamge data"<<std::endl;
	if(plane)
	    header_version = read_frame_header_version(plane);
	else
	    header_version = read_frame_header_version(img);
	    
    }

    if(val == 1){
	std::cerr << "Header frame number for frame 0 is "<<head.nframe<<". Reset frame counter? [yn]";
	char c;
	while(1){
	    std::cin.get(c);
	    if(c == 'y' || c == 'Y'){
		reset_frame_counter = true;
		break;
	    }
	    else if(c == 'n' || c == 'N'){
		reset_frame_counter = false;
		break;
	    }
	}
    }
    
    if(!access (argv[2], F_OK)){
	std::cerr << "File "<< argv[2] << " exists. Continue? [yn]: ";
	char c;

	std::cin.ignore(std::numeric_limits<std::streamsize>::max(),'\n');

	std::cin.get(c);
	if(c != 'y' && c != 'Y')
	    exit(0);	
    }


    f = fopen(argv[2], "w");
    if(!f){
	std::cerr << "Couldn't open RAW file:" << strerror(errno) << std::endl;
	exit(0);
    }

    //if there is a header, check the colorModel is the same in the
    //header and in the img
    if(val>=0){
	if(!CV_IS_IMAGE(plane)){
	    if(img->nChannels >1  && !strcmp(head.color_space, "GRAY")){
		//make a conversion
		convert_color = true;	    
	    }
	}
	else{
	    convert_color = true;
	}
	    
    }


    if(header_version < 2){
	t_offset = (unsigned int) (1e6 / cvGetCaptureProperty(capture, CV_CAP_PROP_FPS)); //tiempo en usec entre frames
	gettimeofday(&tv, NULL);
    }


    if(write_header){

	head.start_code = 0xFF00;
	head.version = 2;
	head.nframe = 0;
	head.counter = t_offset;
	head.width = img->width;
	head.height = img->height;
	head.ncameras = 1;	
	strcpy(head.color_space, img->colorModel);
	head.fps = (unsigned int)cvGetCaptureProperty(capture, CV_CAP_PROP_FPS);
	head.shutter_usec = 0;
	head.gain = 0;
	head.brightness = 0;
	head.lighting = 0;
	head.filltime.tv_sec = tv.tv_sec;
	head.filltime.tv_usec = tv.tv_usec;
    }    

 
    if(write_header || plane){
	if(convert_color){
	    if(plane)
		aux = cvCloneImage(plane);
	}	
	else
	    aux = cvCloneImage(img);
    }
    else 
	aux = NULL;
    

    for(nframe = 0; ; nframe++){
	
	std::cout << "\rFrame: " << nframe << "            ";
	std::cout.flush();

	if(write_header){
	    head.nframe = nframe;

	    tv.tv_usec += t_offset;
	    tv.tv_sec += (tv.tv_usec / 1000000);
	    tv.tv_usec = tv.tv_usec % 1000000;

	    head.filltime.tv_sec = tv.tv_sec;
	    head.filltime.tv_usec = tv.tv_usec;

	    memcpy(aux->imageData, &head, sizeof(struct frame_headerv2));
	    fwrite(aux->imageData, aux->imageSize, 1, f);
	}
	else if(reset_frame_counter){
	    check_frame_header(aux, nframe, &head);
	    head.nframe = nframe;

	    //check the fps value and the counter value are similar:
	    //overwrite the value of the fps
	    if(fabs(head.fps - 1e6/head.counter)>1){
		head.fps = (unsigned int)(1e6/head.counter);
	    }
	    
	    //if the header actually was v1, add the t_offset
	    if(header_version < 2){
		    tv.tv_usec += head.counter;
		    tv.tv_sec += tv.tv_usec / 1000000;
		    tv.tv_usec = tv.tv_usec % 1000000;
		    
		    head.filltime.tv_sec = tv.tv_sec;
		    head.filltime.tv_usec = tv.tv_usec;		
	    }
	    
	    memcpy(aux->imageData, &head, sizeof(struct frame_headerv2));
	    fwrite(aux->imageData, aux->imageSize, 1, f);
	}
	else if(CV_IS_IMAGE(aux))
	    fwrite(aux->imageData, aux->imageSize, 1, f);
	else
	    fwrite(img->imageData, img->imageSize, 1, f);
	
	img = cvQueryFrame(capture);
	if(!CV_IS_IMAGE(img)){
	    std::cout << "End of video"<<std::endl;
	    break;
	}
	if(aux){
	    if(convert_color){
		if(CV_IS_IMAGE(plane)){
		    cvSplit(img, plane, NULL,NULL,NULL);
		    cvCopy(plane, aux);
		}
		else
		    cvCvtColor(img, aux, CV_BGR2GRAY);
	    }
	    else
		cvCopy(img, aux);
	}
    }

    if(CV_IS_IMAGE(plane))
	cvReleaseImage(&plane);
    if(CV_IS_IMAGE(aux))
	cvReleaseImage(&aux);

    cvReleaseCapture(&capture);
    fclose(f);

    return 0;
}
