/*
 * Copyright (c) 2009 Jess Nuevo Chiquero - UAH
 *
 * This file is part of RawVideoTools
 *
 * RawVideoTools is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * RawVideoTools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef _BASIC_UTILS_HH_
#define _BASIC_UTILS_HH_

#include <string>
#include <sstream>
#include <iostream>

#include <cxcore.h>
#include <cv.h>
#include <highgui.h>


template <class T>
bool from_string(T& t, 
                 const std::string& s, 
                 std::ios_base& (*f)(std::ios_base&) = std::dec)
{
  std::istringstream iss(s);
  return !(iss >> f >> t).fail();
}

template <typename T>
std::string to_string(const T &thing) {
    std::stringstream os;
    os << thing;
    return os.str();
}



// /** 
//  * Conversion is performed in-place.
//  * 
//  * @param img 
//  * @param depth 
//  * @param nChannels 
//  */
// void adapt_channels_depth(IplImage **src_img, int depth, int nChannels)
// {

//     IplImage *img = *src_img;

//     if(img->depth == depth && img->nChannels == nChannels)
// 	return;
    
//     if(nChannels != img->nChannels ){
// 	IplImage *aux = cvCreateImage(cvGetSize(img), img->depth, nChannels);
// 	int conv;
// 	if(nChannels == 3) conv = CV_GRAY2BGR;
// 	else conv = CV_BGR2GRAY;
// 	cvCvtColor(img, aux, conv);	    	    		
// 	cvReleaseImage(&img);
// 	img = aux;
//     }
//     if(depth != img->depth){
// 	IplImage *aux = cvCreateImage(cvGetSize(img), depth, nChannels);
// 	cvConvert(img, aux);
// 	cvReleaseImage(&img);
// 	img = aux;
//     }
   
//     *src_img = img;

//     return;
// }


void deleteIplImage(IplImage *img);
void deleteCvMat(CvMat *mat);

void deleteFileStorage(CvFileStorage *fs);

void deleteVideoWriter(CvVideoWriter *vw);

void deleteCvKalman(CvKalman *kalman);

void print_arr(const CvArr *arr, bool transpose = false);


bool is_box_in_image(const CvRect r, const IplImage *img);


#endif
