/*
 * Copyright (c) 2009 Jess Nuevo Chiquero - UAH
 *
 * This file is part of RawVideoTools
 *
 * RawVideoTools is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * RawVideoTools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "frame_header.hh"


// static void headerv1tov2(struct frame_headerv1* v1, struct frame_headerv2 *v2)
// {

//     memcpy(v2, v1, sizeof(struct frame_headerv1));
    
//     v2->version = 2;
//     v2->filltime.tv_sec = 0;
//     v2->filltime.tv_usec = v1->counter;
    
// }


static void headerv1tovn( frame_headerv1* v1, frame_header *v2)
{

    bzero(v1+1, sizeof(v2) - sizeof(v1));
    memcpy(v2, v1, sizeof( struct frame_headerv1));
    
    v2->version = LAST_VERSION; 
}

// static void check_filltime64(frame_header *fh)
// {

// #if __WORDSIZE == 64

//     if(fh->version >1 && fh->version<= 3){
// 	int *ft = (int*)&(fh->filltime);
// 	timeval32 tv;

// 	tv.tv_sec = ft[0];
// 	tv.tv_usec = ft[1];

// 	fh->filltime = tv;

// 	if(fh->version == 3){
// 	    fh->aux = ft[2];
// 	}
//     }
// #endif    

// }

bool check_frame_header(frame_header &head)
{
	return head.start_code == FRAME_START_CODE;
}


int read_frame_header_version(FILE *f)
{

    off_t curr;
    curr = ftello(f);

    unsigned int start_code;
    unsigned int version;

    fread(&start_code, sizeof(uint), 1, f);
    if(start_code != FRAME_START_CODE){
	fseeko(f, curr, SEEK_SET);
	fprintf(stderr, "Bad header: bad START_CODE\n");
	return -1;
    }

    fread(&version, sizeof(uint), 1, f);
    
    fseeko(f,curr, SEEK_SET);

    return version;
}

int read_frame_header_version(const IplImage *img)
{

    struct frame_headerv1 h;

    memcpy(&h, img->imageData, sizeof(struct frame_headerv1)); 

    if(h.start_code != FRAME_START_CODE){
	fprintf(stderr, "Bad header: bad START_CODE\n");
	return -1;
    }

    return h.version;
}

int read_frame_header(FILE *f, frame_header *head)
{

    struct frame_headerv1 v1;
    off_t curr;
    curr = ftello(f);

    v1.start_code = 0;    

    fread(&v1, sizeof(struct frame_headerv1), 1, f); 
    fseeko(f, curr, SEEK_SET);

    if(v1.start_code != FRAME_START_CODE){
	fprintf(stderr, "Bad header: bad START_CODE\n");
	return -1;
    }
    if(v1.version ==2){
	fread(head, sizeof(struct frame_headerv2), 1, f); 	
	fseeko(f, curr, SEEK_SET);
	head->aux = 0;
    }
    else{
	fread(head, sizeof(frame_header), 1, f); 	
	fseeko(f, curr, SEEK_SET);
	//headerv1tovn(&v1,head);
    }
     
    //    check_filltime64(head);

// #if __WORDSIZE == 64
//     if(v1.version == 3){
// 	struct frame_headerv3_64 fh64;
// 	fread(&fh64,sizeof(frame_headerv3_64), 1, f); 	
// 	fseeko(f, curr, SEEK_SET);
	

//     }
    
// #endif    


    return 0;
}


int read_frame_header(FILE *f, struct frame_headerv2 *head)
{

    struct frame_headerv1 v1;
    off_t curr;
    curr = ftello(f);

    v1.start_code = 0;    

    fread(&v1, sizeof(struct frame_headerv1), 1, f); 
    fseeko(f, curr, SEEK_SET);

    if(v1.start_code != FRAME_START_CODE){
	fprintf(stderr, "Bad header: bad START_CODE\n");
	return -1;
    }
    if(v1.version == 2){
	fread(head, sizeof(struct frame_headerv2), 1, f); 
	fseeko(f, curr, SEEK_SET);
    }

//     check_filltime64((frame_header*)head);
    
    return 0;
}

int read_frame_header(FILE *f, struct frame_headerv1 *head)
{

    off_t curr;

    curr = ftello(f);

    head->start_code = 0;

    fread(head, sizeof(struct frame_headerv1), 1, f); 

    fseeko(f, curr, SEEK_SET);
    
    if(head->start_code != FRAME_START_CODE){
	fprintf(stderr, "Bad header: bad START_CODE\n");
	return -1;
    }

//     if(head->version == 1){
//  	head->version = 1;
//    }    


    return 0;   
}




int check_frame_header(const IplImage *img, unsigned int frame, 
		       struct frame_headerv1 *hout)
{

    //check the first bytes are the FRAME_START_CODE
    
    int code = *((int*)img->imageData);
    if(code != FRAME_START_CODE)
	return -1;    

    //and that size and frame are correct
    struct frame_headerv1 h;

    memcpy(&h, img->imageData, sizeof(struct frame_headerv1)); 
    if(hout){
	memcpy(hout, img->imageData, sizeof(struct frame_headerv1)); 
    }


//     sscanf(img->imageData, "%4c %d %d %d %d %d %d", h.start_code, &(h.version), &h.nframe,
// 	   &(h.counter), &(h.width), &(h.height), &(h.ncameras));

    if(frame != h.nframe)
	return 1;
    
    if((unsigned int)img->width != h.width || (unsigned int)img->height!=(h.height*h.ncameras))
	return -2;
    
    return 0;
}

int check_frame_header(const IplImage *img, unsigned int frame, 
		       frame_header *hout)
{

    //check the first bytes are the FRAME_START_CODE
    
    int code = *((int*)img->imageData);
    if(code != FRAME_START_CODE)
	return -1;    
    
    //and that size and frame are correct
    struct frame_headerv1 h;
    
    memcpy(&h, img->imageData, sizeof(struct frame_headerv1)); 
    if(hout){
	if(h.version==1){
	    headerv1tovn(&h, hout);
	}
	else
	    memcpy(hout, img->imageData, sizeof(struct frame_headerv2)); 
    }
    
    
    //     sscanf(img->imageData, "%4c %d %d %d %d %d %d", h.start_code, &(h.version), &h.nframe,
    // 	   &(h.counter), &(h.width), &(h.height), &(h.ncameras));
    
    if(frame != h.nframe)
	return 1;
    
    if(h.ncameras==0){
	h.ncameras = 1;
    }
    if((unsigned int)img->width != h.width || (unsigned int)img->height!=(h.height*h.ncameras))
	return -2;
    
    return 0;
}
