/*
 * Copyright (c) 2009 Jess Nuevo Chiquero - UAH
 *
 * This file is part of RawVideoTools
 *
 * RawVideoTools is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * RawVideoTools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef _FRAME_HEADER_HH_
#define _FRAME_HEADER_HH_

#include <stdio.h>

#include "cxcore.h"

#define FRAME_START_CODE 0x0000FF00U

typedef struct timeval32{
    int32_t tv_sec;
    int32_t tv_usec;
}timeval32;


typedef struct frame_headerv1{
    unsigned int start_code; //0xFF00
    unsigned int version;
    unsigned int nframe;
    unsigned int counter;
    unsigned int width;
    unsigned int height;
    unsigned int ncameras;
    char color_space[4];
    unsigned int fps;
    unsigned int shutter_usec;
    unsigned int gain;
    unsigned int brightness;
    unsigned int lighting;
} frame_headerv1;

typedef struct frame_headerv2{
    unsigned int start_code; //0xFF00
    unsigned int version; // 2
    unsigned int nframe;
    unsigned int counter;
    int width;
    int height;
    unsigned int ncameras;
    char color_space[4];
    unsigned int fps;
    unsigned int shutter_usec;
    unsigned int gain;
    unsigned int brightness;
    unsigned int lighting;
    timeval32 filltime;
} frame_headerv2;

typedef struct frame_headerv3{
    unsigned int start_code; //0xFF00
    unsigned int version; // 3
    unsigned int nframe;  //frame counter
    unsigned int counter;  //sincrhonization tics counter
    int width;  //image width
    int height; //image height
    unsigned int ncameras;  //camera index among all the working cameras 
    char color_space[4];  //frame color format: usually raw8= RGB bayer pattern
    unsigned int fps;  //estimated frame rate
    unsigned int shutter_usec; //exposition time in micro secs
    unsigned int gain;  //camera gain in camera units
    unsigned int brightness; 
    unsigned int lighting;  //ilumination intensity in %

    timeval32 filltime;

    int aux;  //user calibration information
} frame_headerv3;

typedef struct frame_headerv3_64{
    unsigned int start_code; //0xFF00
    unsigned int version; // 3
    unsigned int nframe;  //frame counter
    unsigned int counter;  //sincrhonization tics counter
    int width;  //image width
    int height; //image height
    unsigned int ncameras;  //camera index among all the working cameras 
    char color_space[4];  //frame color format: usually raw8= RGB bayer pattern
    unsigned int fps;  //estimated frame rate
    unsigned int shutter_usec; //exposition time in micro secs
    unsigned int gain;  //camera gain in camera units
    unsigned int brightness; 
    unsigned int lighting;  //ilumination intensity in %

    timeval filltime;

    int aux;  //user calibration information
} frame_headerv3_64;


typedef frame_headerv3 frame_header;
#define LAST_VERSION 3

int read_frame_header_version(FILE *f);
int read_frame_header_version(const IplImage *img);
int read_frame_header(FILE *f, struct frame_headerv1 *head);
int read_frame_header(FILE *f, struct frame_headerv2 *head);
int read_frame_header(FILE *f, struct frame_headerv3 *head);

frame_header * read_image(IplImage * img, FILE * f, CvSize video_size);
int check_frame_header(const IplImage *img, unsigned int frame, struct frame_headerv1* hout);
int check_frame_header(const IplImage *img, unsigned int frame, frame_header* hout);

void init_head(frame_header * head, int colors=1);
// int write_frame_header(const IplImage *img, int nframe, int ncameras, int fps);

#endif
